/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.jdbc.dao;

import org.wso2.registry.jdbc.DatabaseConstants;
import org.wso2.registry.jdbc.dataobjects.RatingDO;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

public class RatingsDAO {

    public void addRating(long artifactID, String userID, int rating, Connection conn)
            throws SQLException {

        String sql = "INSERT INTO RATINGS (AID, USER_ID, RATING, RATED_TIME) " +
                     "VALUES (?, ?, ?, ?)";

        long now = System.currentTimeMillis();

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, artifactID);
        s.setString(2, userID);
        s.setInt(3, rating);
        s.setDate(4, new Date(now));

        s.executeUpdate();
    }

    public void copyRatings(long fromResourceID , long toResourceId , Connection conn) throws SQLException {
        String SQL  = "SELECT USER_ID , RATING , RATED_TIME FROM RATINGS WHERE AID=?";
        PreparedStatement s = conn.prepareStatement(SQL);
        s.setLong(1, fromResourceID);

        ResultSet result = s.executeQuery();
        ArrayList list = new ArrayList();
        if (result.next()) {
            RatingDO rate = new RatingDO();
            rate.setRatedUserName(result.getString(1));
            rate.setRating(result.getInt(2));
            rate.setRatedTime(result.getDate(3));
            list.add(rate);
        }
        for (int i = 0; i < list.size(); i++) {
            RatingDO ratingDO = (RatingDO) list.get(i);
            addRating(toResourceId , ratingDO.getRatedUserName() ,ratingDO.getRating() ,conn);
        }
    }

    public void updateRating(long artifactID, String userID, int rating, Connection conn)
            throws SQLException {

        String sql = "UPDATE RATINGS SET RATING=?, RATED_TIME=? WHERE AID=? AND USER_ID=?";

        long now = System.currentTimeMillis();

        PreparedStatement s = conn.prepareStatement(sql);
        s.setInt(1, rating);
        s.setDate(2, new Date(now));
        s.setLong(3, artifactID);
        s.setString(4, userID);

        s.executeUpdate();
    }

    public boolean ratingExist(long artifactID, String userID, Connection conn)
            throws SQLException {

        String sql = "SELECT COUNT(R.R_ID) FROM RATINGS R WHERE R.AID=? AND R.USER_ID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, artifactID);
        s.setString(2, userID);

        ResultSet result = s.executeQuery();
        long count = 0;
        if (result.next()) {
            count = result.getLong(1);
        }

        return count == 1;
    }

    public float getAverageRating(String path, Connection conn) throws SQLException {

        String sql1 =
                "SELECT SUM(R.RATING) FROM RATINGS R, ARTIFACTS A WHERE A.PATH=? AND A.AID=R.AID";

        PreparedStatement s1 = conn.prepareStatement(sql1);
        s1.setString(1, path);

        ResultSet result1 = s1.executeQuery();

        int sumRating = 0;
        if (result1.next()) {
            sumRating = result1.getInt(1);
        }

        String sql2 =
                "SELECT COUNT(R.RATING) FROM RATINGS R, ARTIFACTS A WHERE A.PATH=? AND A.AID=R.AID";

        PreparedStatement s2 = conn.prepareStatement(sql2);
        s2.setString(1, path);

        ResultSet result2 = s2.executeQuery();

        int countRating = 0;
        if (result2.next()) {
            countRating = result2.getInt(1);
        }

        float averageRating = 0;
        if (countRating > 0) {
            averageRating = (float)sumRating / countRating;
        }

        return averageRating;
    }

    public int getRating(String path, String userName, Connection conn) throws SQLException {

        String sql =
                "SELECT R.RATING FROM RATINGS R, ARTIFACTS A WHERE A.PATH=? AND R.USER_ID=? AND A.AID=R.AID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, path);
        s.setString(2, userName);

        ResultSet result = s.executeQuery();

        int rating = 0;
        if (result.next()) {
            rating = result.getInt(1);
        }

        return rating;
    }

    public java.util.Date getRatedTime(String path, String userName, Connection conn)
            throws SQLException {

        String sql =
                "SELECT R.RATED_TIME FROM RATINGS R, ARTIFACTS A WHERE A.PATH=? AND R.USER_ID=? AND A.AID=R.AID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, path);
        s.setString(2, userName);

        ResultSet result = s.executeQuery();

        java.util.Date ratedTime = null;
        if (result.next()) {
            ratedTime = new java.util.Date(
                    result.getTimestamp(DatabaseConstants.RATED_TIME_FIELD).getTime());
        }

        return ratedTime;
    }

    public void removeRatings(long artifactID, Connection conn) throws SQLException {

        String sql = "DELETE FROM RATINGS WHERE AID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, artifactID);
        s.executeUpdate();
    }

    public String[] getRatedUserNames(String resourcePath, Connection conn) throws SQLException {

        String sql = "SELECT R.USER_ID FROM RATINGS R, ARTIFACTS A WHERE A.PATH=? AND A.AID=R.AID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, resourcePath);

        List userNameList = new ArrayList();
        ResultSet results = s.executeQuery();
        while (results.next()) {
            String ratedUserName = results.getString(DatabaseConstants.USER_ID_FIELD);
            userNameList.add(ratedUserName);
        }

        String[] userNames = new String[userNameList.size()];
        for (int i = 0; i < userNames.length; i++) {
            userNames[i] = ((String)userNameList.get(i));
        }

        return userNames;
    }

    public RatingDO getRating(long ratingID, Connection conn) throws SQLException {

        String sql =
                "SELECT A.PATH, R.AID, R.USER_ID, R.RATING, R.RATED_TIME FROM RATINGS R, ARTIFACTS A WHERE A.AID=R.AID AND R.R_ID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, ratingID);

        ResultSet results = s.executeQuery();
        if (results.next()) {

            java.util.Date ratedTime = new java.util.Date(
                    results.getTimestamp(DatabaseConstants.RATED_TIME_FIELD).getTime());

            RatingDO ratingDAO = new RatingDO();
            ratingDAO.setRatedUserName(results.getString(DatabaseConstants.USER_ID_FIELD));
            ratingDAO.setRatedTime(ratedTime);
            ratingDAO.setRating(results.getInt(DatabaseConstants.RATING_FIELD));
            ratingDAO.setResourcePath(results.getString(DatabaseConstants.PATH_FIELD));
            ratingDAO.setResourceID(results.getLong(DatabaseConstants.AID_FIELD));

            return ratingDAO;
        }

        return null;
    }
}
